package com.xhkjedu.sstudy.task;

import com.xhkjedu.sstudy.utils.StudyUtil;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import java.util.concurrent.ConcurrentHashMap;

/**
 * @author ywx
 * @classname ReadTask
 * @description 阅读定时任务
 * CronTrigger配置完整格式为： [秒] [分] [小时] [日] [月] [周] [年]
 * 说明    必填    值                  允许的通配符
 * 秒        是    0-59                , - * /
 * 分        是    0-59                , - * /
 * 时        是    0-23                , - * /
 * 日        是    1-31                , - * ? / L W
 * 月        是    1-12或JAN-DEC       , - * /
 * 周        是    1-7或SUN-SAT        , - * ? / L W
 * 年        否    empty 或1970-2099   , - * /
 * , 表示指定多个值，例如在周字段上设置 "MON,WED,FRI" 表示周一，周三和周五触发
 * - 表示区间。例如 在小时上设置 "10-12",表示 10,11,12点都会触发。
 * * 表示所有值. 例如:在分的字段上设置 "*",表示每一分钟都会触发
 * ? 表示不指定值。使用的场景为不需要关心当前设置这个字段的值。每月的10号触发一个操作，但不关心是周几,具体设置为 0 0 0 10 * ?
 * / 用于递增触发。如在秒上面设置"5/15" 表示从5秒开始，每增15秒触发(5,20,35,50)
 * L 表示最后的意思。在日字段设置上，表示当月的最后一天(依据当前月份，如果是二月还会依据是否是润年[leap]),
 * 在周字段上表示星期六，相当于"7"或"SAT(不区分大小写)"。
 * 如果在"L"前加上数字，则表示该数据的最后一个。例如在周字段上设置"6L"这样的格式,则表示“本月最后一个星期五"
 * W 表示离指定日期的最近那个工作日(周一至周五). 例如在日字段上设置"15W"，表示离每月15号最近的那个工作日触发。
 * 如果15号正好是周六，则找最近的周五(14号)触发, 如果15号是周未，则找最近的下周一(16号)触发.
 * 如果15号正好在工作日(周一至周五)，则就在该天触发。(注，"W"前只能设置具体的数字,不允许区间"-").
 * @date 2020/9/1 9:36
 **/
@Component
public class UriMapTask {

    /**
     * 每隔3s清除一下请求地址map
     * @Param []
     * @Author ywx
     * @Date 2020/11/9 12:30
     * @return void
     **/
    @Scheduled(cron = "${cron.deleteUriMap}")
    public void deleteUriMap() {
        StudyUtil.uriMap = new ConcurrentHashMap<>();
    }
}
