package com.xhkjedu.suser.utils;

import com.alibaba.fastjson.JSON;
import com.xhkjedu.suser.config.ConfigKey;
import com.xhkjedu.utils.MD5;
import com.xhkjedu.utils.N_Utils;
import com.xhkjedu.suser.vo.system.ZtMsgVo;
import lombok.extern.slf4j.Slf4j;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.net.URL;
import java.net.URLConnection;

@Slf4j
public class SMSSender {
    /**
     * 发送短信
     * @return java.lang.String
     * @Param [msg]
     * @Author ywx
     * @Date 2020/12/11 16:15
     **/
    public static String sendSms(ZtMsgVo msg) {
        msg.setUsername(ConfigKey.username);
        Integer timestamp = N_Utils.getSecondTimestamp();
        String tKey = timestamp.toString();
        msg.setTKey(tKey);
        MD5 m = new MD5();
        String password = m.getMD5ofStr2(m.getMD5ofStr2(ConfigKey.password) + tKey);
        msg.setPassword(password);
        msg.setContent(ConfigKey.signName + msg.getContent());
        String param = JSON.toJSONString(msg);
        return sendPost(ConfigKey.url + "v2/sendSms", param);
    }

    /**
     * 向指定 URL 发送POST方法的请求
     * @param url   发送请求的 URL
     * @param param 请求参数，请求参数应该是 name1=value1&name2=value2 的形式。
     * @return 所代表远程资源的响应结果
     */
    public static String sendPost(String url, String param) {
        OutputStreamWriter out = null;
        BufferedReader in = null;
        String result = "";
        try {
            URL realUrl = new URL(url);
            // 打开和URL之间的连接
            URLConnection conn = realUrl.openConnection();
            // 设置通用的请求属性
            conn.setRequestProperty("accept", "application/json");
            conn.setRequestProperty("connection", "Keep-Alive");
            conn.setRequestProperty("user-agent", "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1;SV1)");
            // 发送POST请求必须设置如下两行
            conn.setDoOutput(true);
            conn.setDoInput(true);
            //设置请求编码格式
            conn.setRequestProperty("Accept-Charset", "UTF-8");
            conn.setRequestProperty("Content-Type", "application/json; charset=UTF-8");
            // 文件流编码设置
            out = new OutputStreamWriter(conn.getOutputStream(), "UTF-8");
            out.write(param);
            // flush输出流的缓冲
            out.flush();
            // 定义BufferedReader输入流来读取URL的响应
            in = new BufferedReader(new InputStreamReader(conn.getInputStream(), "UTF-8"));
            String line;
            while ((line = in.readLine()) != null) {
                result += line;
            }
        } catch (Exception e) {
            log.error("发送 POST 请求出现异常！" + e.getMessage());
        }
        // 使用finally块来关闭输出流、输入流
        finally {
            try {
                if (out != null) {
                    out.close();
                }
                if (in != null) {
                    in.close();
                }
            } catch (IOException ex) {
                log.error("关闭输入输出流异常", ex.getMessage());
            }
        }
        return result;
    }
}
